﻿using System;
using System.IO;
using System.Net;
using System.Text;
using System.Collections.Generic;
using CMS.SettingsProvider;
using CMS.CMSHelper;

namespace SMSNotification
{
    /// <summary>
    /// Base class for sms provider.
    /// </summary>
    public class SMSProvider
    {
        #region "Contants"

        private const string USER_AGENT = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; .NET CLR 1.0.3705;)";

        #endregion

        #region "Variables"

        private string mAthentificationError = null;
        private string mMessageSendError = null;
        private bool mAuthenticated = false;
        private bool mMessageSended = false;
        private string mSessionID = null;
        private string mLogin = null;
        private string mPassword = null;
        private string mApiID = null;
        private int mMaxMessageSplit = -1;

        #endregion

        #region "Properties"

        /// <summary>
        /// Athentification result.
        /// </summary>
        public string AthentificationError
        {
            get
            {
                return this.mAthentificationError;
            }
        }


        /// <summary>
        /// Message send result.
        /// </summary>
        public string MessageSendError
        {
            get
            {
                return this.mMessageSendError;
            }
        }


        /// <summary>
        /// Indicates if is authenticated.
        /// </summary>
        public bool Authenticated
        {
            get
            {
                return this.mAuthenticated;
            }
        }


        /// <summary>
        /// Indicates if message was sended.
        /// </summary>
        public bool MessageSended
        {
            get
            {
                return this.mMessageSended;
            }
        }


        /// <summary>
        /// Session ID for sending message.
        /// </summary>
        public string SessionID
        {
            get
            {
                return this.mSessionID;
            }
        }


        /// <summary>
        /// Login for clicatell.com account.
        /// </summary>
        public string Login
        {
            get
            {
                if (this.mLogin == null)
                {
                    string siteName = (CMSContext.CurrentSiteName != null ? CMSContext.CurrentSiteName : "");
                    this.mLogin = SettingsKeyProvider.GetStringValue(siteName + ".SMSLogin");
                }
                return this.mLogin;
            }
            set
            {
                this.mLogin = value;
            }
        }


        /// <summary>
        /// Password for clicatell.com account.
        /// </summary>
        public string Password
        {
            get
            {
                if (this.mPassword == null)
                {
                    string siteName = (CMSContext.CurrentSiteName != null ? CMSContext.CurrentSiteName : "");
                    this.mPassword = SettingsKeyProvider.GetStringValue(siteName + ".SMSPassword");
                }
                return this.mPassword;
            }
            set
            {
                this.mPassword = value;
            }
        }


        /// <summary>
        /// Api ID for clicatell.com account.
        /// </summary>
        public string ApiID
        {
            get
            {
                if (this.mApiID == null)
                {
                    string siteName = (CMSContext.CurrentSiteName != null ? CMSContext.CurrentSiteName : "");
                    this.mApiID = SettingsKeyProvider.GetStringValue(siteName + ".SMSApiID");
                }
                return this.mApiID;
            }
            set
            {
                this.mApiID = value;
            }
        }


        /// <summary>
        /// Number of max SMS messages that will be sended if message is long.
        /// </summary>
        public int MaxMessageSplit
        {
            get
            {
                if (this.mMaxMessageSplit == -1)
                {
                    string siteName = (CMSContext.CurrentSiteName != null ? CMSContext.CurrentSiteName : "");
                    this.mMaxMessageSplit = SettingsKeyProvider.GetIntValue(siteName + ".SMSMaxMessageSplit");
                }
                return this.mMaxMessageSplit;
            }
            set
            {
                this.mMaxMessageSplit = value;
            }
        }

        #endregion

        #region "Public methods"

        /// <summary>
        /// Authenticate into Clicatell.com account and sets Session ID.
        /// </summary>
        public void Authenticate()
        {
            Authenticate(Login, Password, ApiID);
        }


        /// <summary>
        /// Authenticate into Clicatell.com account and sets Session ID.
        /// </summary>
        /// <param name="login">Login</param>
        /// <param name="password">Password</param>
        /// <param name="apiId">Api ID</param>
        public void Authenticate(string login, string password, string apiId)
        {
            WebClient client = new WebClient();
            // Add a user agent header in case the requested URI contains a query.
            client.Headers.Add("user-agent", USER_AGENT);
            client.QueryString.Add("user", login);
            client.QueryString.Add("password", password);
            client.QueryString.Add("api_id", apiId);
            string baseurl = "https://api.clickatell.com/http/auth";
            Stream data = client.OpenRead(baseurl);
            StreamReader reader = new StreamReader(data);
            string response = reader.ReadToEnd();
            data.Close();
            reader.Close();

            if (response != null)
            {
                if (response.StartsWith("ok", StringComparison.CurrentCultureIgnoreCase))
                {
                    // Authentication OK
                    mSessionID = response.Substring(4);

                    // Set authentication info
                    mAuthenticated = true;
                }
                else
                {
                    response = response.Substring(5);
                    string[] err = response.Split(',');
                    mAuthenticated = false;
                    mAthentificationError = "Error number: " + err[0] + " Error description: " + err[1];
                }
            }
            else
            {
                mAuthenticated = false;
                mAthentificationError = "Error: No response from clickatell.com";
            }
        }


        /// <summary>
        /// Send SMS message.
        /// </summary>
        /// <param name="message">SMS message object</param>
        public void SendSMS(SMSMessage message)
        {
            if (message != null)
            {
                // Check if athenticated
                if (!this.Authenticated)
                {
                    Authenticate();
                    // If still not authenticated return
                    if (!this.Authenticated)
                    {
                        mMessageSended = false;
                        mMessageSendError = this.AthentificationError;
                        return;
                    }
                }

                WebClient client = new WebClient();
                // Add a user agent header in case the requested URI contains a query.
                client.Headers.Add("user-agent", USER_AGENT);
                client.QueryString.Add("session_id", mSessionID);
                client.QueryString.Add("to", message.To);
                if (!String.IsNullOrEmpty(message.From))
                {
                    client.QueryString.Add("from", message.From);
                }
                if (this.MaxMessageSplit > 1)
                {
                    client.QueryString.Add("concat", this.MaxMessageSplit.ToString());
                }
                client.QueryString.Add("text", message.Text);
                string baseurl = "https://api.clickatell.com/http/sendmsg";
                Stream data = client.OpenRead(baseurl);
                StreamReader reader = new StreamReader(data);
                string response = reader.ReadToEnd();
                data.Close();
                reader.Close();
                if (response != null)
                {
                    if (response.StartsWith("id", StringComparison.CurrentCultureIgnoreCase))
                    {
                        mMessageSended = true;
                    }
                    else
                    {
                        response = response.Substring(5);
                        string[] err = response.Split(',');
                        mMessageSended = false;
                        mMessageSendError = "Error number: " + err[0] + " Error description: " + err[1];
                    }
                }
                else
                {
                    mMessageSended = false;
                    mMessageSendError = "Error: No response from clickatell.com";
                }
            }
            else
            {
                mMessageSended = false;
                mMessageSendError = "Error: No message present.";
            }
        }

        #endregion
    }
}
